import { DataSet, Network } from "vis/index-network";
import workService from '../../services/fos/work.service.js'

let workHttps = new workService()
let ori_data = null,
    cur_data = {},
    network = null,
    container = null,
    nodeBorderWidth = 5,
    vm = null,
    options = {
        physics: {
            enabled: false
        },
        interaction: { hover: true },
        nodes: {
            shape: "dot",
            size: 10,
            font: {
                size: 13
            },
            borderWidth: nodeBorderWidth
        },
        edges: {
            width: 1,
            smooth: false,
            font: {
                size: 10
            }
        }
    };

let bExcIds = { "0": [], "1": [], "2": [] };
let int = 0;
let nodeout = null
let nodecontext = null;
let edgecontext = null;
let colors = {
    n: "#dddddd", // 闪烁背景色
    y: "#ffff00", // 黄色   (闪烁背景色)
    d: "#cf2b0b", // 深红   (闪烁背景色)
    l: "#59db49", // 线默认色
    r: "#ff3d32", // 红色   (节点出问题)
    q:'#72D572',  // 淡绿色
    g: "#0A8F08", // 绿色
    b: '#2CABE3', // 蓝色

};
// 0 异常,1 正常,2 黄色报警,3 橙色报警,4 红色报警,5 紫色报警
let levels = ["a", "l", "y", "o", "d", "v"];

let MapNodeIdIndex = {};
let MapEdgeIdIndex = {};

let registerNetworkEvent = () => {
    network.on("zoom", params => {
        let scale = network.getScale();
        if (scale < 0.3) {
            scale = 0.3;
        }
        network.moveTo({ scale: scale });
      localStorage.setItem("scale", JSON.stringify(scale));
      localStorage.setItem(
        "viewposition",
        JSON.stringify(network.getViewPosition())
      );
    });
    //右击事件
    network.on("oncontext", params => {

    });
    //屏蔽右键菜单
    document.getElementsByTagName("canvas")[0].oncontextmenu = function(ev) {
        ev.preventDefault();
    };
    window.Event.prototype.preventDefault = window.Event.prototype.preventDefault || function(){
        //阻止冒泡
        this.cancelBubble = true;
        //阻止触发默认事件
        this.returnValue = false;
    }
    window.event.oncontextmenu = function(ev) {
        ev.preventDefault();
    };
    //双击事件
    network.on("doubleClick", params => {

    });

    //更新闪烁状态
    // int = setInterval(() => {
    //     update_nodes_status();
    // }, 1000);
    //屏蔽右键菜单
    document.getElementsByTagName("canvas")[0].oncontextmenu = function(ev) {
        ev.preventDefault();
    };
};
let moveTo = () => {
    let scale = JSON.parse(localStorage.getItem("scale"));
    let viewposition = JSON.parse(localStorage.getItem("viewposition"));
    if (scale && viewposition) {
        network.moveTo({
            position: viewposition,
            scale: scale
        });
    }
};
let  saveviewpos = () => {
    localStorage.setItem("scale", JSON.stringify(network.getScale()));
    localStorage.setItem(
        "viewposition",
        JSON.stringify(network.getViewPosition())
    );
};

let  update_nodes_status = () => {
    if (cur_data["nodes"]) {
        //正常光环的
        if (bExcIds["1"].length > 0) {
            bExcIds["1"].forEach(v => {
                if (v.color.border == colors.n) {
                    v.color.border = colors.y;
                } else if (v.color.border == colors.y) {
                    v.color.border = colors.n;
                }
            })
        }
        cur_data["nodes"].update(bExcIds["1"]);
    }
    if (bExcIds["2"].length > 0) {
        bExcIds["2"].forEach(v => {
            if (v.color.border == colors.n) {
                v.color.border = colors.d;
            } else if (v.color.border == colors.d) {
                v.color.border = colors.n;
            }
        });
        cur_data["nodes"].update(bExcIds["2"]);
    }
};
let  handleData = (data) =>{
    let newData = {nodes:[],edges:[]}
    if(data.retNodes.length !=0){
        data.retNodes.forEach((v,i)=>{
            v.id = v.ip
            v.x = v.axisX
            v.y = v.axisY
            v.label = `${v.nodeName }(${v.ip})`
            newData.nodes.push(v)
        })
    }
    if(data.retLinks.length !=0){
        data.retLinks.forEach((v,i)=>{
            v.id = v.lkId
            v.from = v.snode
            v.to = v.dnode
            newData.edges.push(v)
        })
    }
    return newData
}
export default {
    init: (ccClassName, data,that) => {
        data = handleData(data)
        vm = that
        sessionStorage.setItem("graph", JSON.stringify(data));
        container = document.querySelector(`#${ccClassName}`);  //topu 容器
        ori_data = data;                                        //原始数据
        cur_data = JSON.parse(JSON.stringify(ori_data));        //拷贝数据(使用)
        //status    节点状态       0 异常  1 正常  2 初始状态
        cur_data.nodes.forEach((v, i) => {
            v.color = {}
            if(v.status === '0'){
                v.color.background = colors.g
            }else {
                v.color.background = colors.r
            }
        });
        //遍历 线
        //state 0 正常 1异常
        cur_data.edges.forEach((v, i) => {
            v.color = { color: colors.l };
            v.width = 1;
        });
    },
    getnetwork: () => {
        return network;
    },
    destroy: () => {
        network && (network.destroy(), (network = null));
        container && (container.innerHTML = "");
        int = clearInterval(int);
        ori_data = null;
        cur_data = {};
        int = 0;
        localStorage.removeItem("cfg");
        localStorage.removeItem('scale');
        localStorage.removeItem('viewposition');
    },
    cgraph: (fn, S, __fn) => {
        //改变数据类型
        cur_data["edges"] = new DataSet(cur_data["edges"]);
        cur_data["nodes"] = new DataSet(cur_data["nodes"]);
        //初始绘制图表
        if (!container) return;
        network = new Network(container, cur_data, options);
        network.moveTo({ scale: 0.5 });
        registerNetworkEvent();
        if (!S) {
            //计算位置
            let initpos = network.getPositions();
            let initid = Object.keys(initpos)[0];
            sessionStorage.setItem("initpos", JSON.stringify(initpos[initid]));
            sessionStorage.setItem("initid", initid);
        }
    },
};
